@@ -15,6 +15,7 @@ sudo apt-get install -y --allow-unauthenticated build-essential protobuf-c-compi
 sudo apt-get install -y --allow-unauthenticated libmono-system-drawing4.0-cil mono-mcs
 sudo apt-get install -y --allow-unauthenticated libperl-dev
 sudo apt-get install -y --allow-unauthenticated openjdk-8-jdk
+sudo apt-get install -y --allow-unauthenticated libonig5
 
 #install recent cmake on GH build action
 if [ -z ${TRAVIS+x} ]; then
@@ -25,9 +26,9 @@ else
     # install recent CMake on Travis
     DEPS_DIR="${TRAVIS_BUILD_DIR}/deps"
     mkdir ${DEPS_DIR} && cd ${DEPS_DIR}
-    wget --no-check-certificate https://cmake.org/files/v3.23/cmake-3.23.1-linux-x86_64.tar.gz
-    tar -xvf cmake-3.23.1-linux-x86_64.tar.gz > /dev/null
-    mv cmake-3.23.1-linux-x86_64 cmake-install
+    wget --no-check-certificate https://github.com/Kitware/CMake/releases/download/v3.26.3/cmake-3.26.3-linux-x86_64.tar.gz
+    tar -xvf cmake-3.26.3-linux-x86_64.tar.gz > /dev/null
+    mv cmake-3.26.3-linux-x86_64 cmake-install
     export PATH=${DEPS_DIR}/cmake-install:${DEPS_DIR}/cmake-install/bin:${PATH}
     cd ${TRAVIS_BUILD_DIR}
     export MSBUILD_ENV="TRAVIS"
@@ -78,7 +79,7 @@ sudo service postgresql restart 12
 
 cd msautotest
 #upgrade to recent PHPUnit
-cd php && curl -LO https://phar.phpunit.de/phpunit-9.5.phar
+cd php && curl -LO https://phar.phpunit.de/phpunit-10.phar
 cd ..
 python -m pyflakes .
 ./create_postgis_test_data.sh
