@@ -709,9 +709,18 @@ static json getCollection(mapObj *map, layerObj *layer, OGCAPIFormat format)
 
 static void outputJson(const json& j, const char *mimetype)
 {
+  std::string js;
+
+  try {
+    js = j.dump();
+  } catch (...) {
+    outputError(OGCAPI_CONFIG_ERROR, "Invalid UTF-8 data, check encoding.");
+    return;
+  }
+
   msIO_setHeader("Content-Type", "%s", mimetype);
   msIO_sendHeaders();
-  msIO_printf("%s\n", j.dump().c_str());
+  msIO_printf("%s\n", js.c_str());
 }
 
 static void outputTemplate(const char *directory, const char *filename, const json& j, const char *mimetype)
@@ -720,8 +729,6 @@ static void outputTemplate(const char *directory, const char *filename, const js
   std::string _filename(filename);
   Environment env {_directory}; // catch
 
-  // somehow need to limit include processing to the directory
-
   // ERB-style instead of Mustache (we'll see)
   // env.set_expression("<%=", "%>");
   // env.set_statement("<%", "%>");
@@ -731,6 +738,13 @@ static void outputTemplate(const char *directory, const char *filename, const js
   //   - contains (substring)
   //   - URL encode
 
+  try {
+    std::string js = j.dump();
+  } catch (...) {
+    outputError(OGCAPI_CONFIG_ERROR, "Invalid UTF-8 data, check encoding.");
+    return;
+  }
+
   try {
     Template t = env.parse_template(_filename); // catch
     std::string result = env.render(t, j);
@@ -1058,9 +1072,10 @@ static int processCollectionItemsRequest(mapObj *map, cgiRequestObj *request, co
               return MS_SUCCESS;
             }
 
-            // msExecuteQuery() use a 1-based offst convention, whereas the API
+            // msExecuteQuery() use a 1-based offset convention, whereas the API
             // uses a 0-based offset convention.
             map->query.startindex = 1 + offset;
+            layer->startindex = 1 + offset;
         }
 
         if(msExecuteQuery(map) != MS_SUCCESS) {
