@@ -147,8 +147,14 @@
 #define MS_OWS_ERROR_OPTION_NOT_SUPPORTED       "OptionNotSupported"
 #define MS_OWS_ERROR_NO_APPLICABLE_CODE         "NoApplicableCode"
 
+#define MS_OWS_ERROR_NOT_FOUND                  "NotFound"
+
 #define MS_WFS_ERROR_OPERATION_PROCESSING_FAILED "OperationProcessingFailed"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #ifdef USE_LIBXML2
 
 /* function prototypes */
@@ -172,9 +178,9 @@ xmlNodePtr msOWSCommonOperationsMetadata(xmlNsPtr psNsOws);
 #define OWS_METHOD_POST    2
 #define OWS_METHOD_GETPOST 3
 
-xmlNodePtr msOWSCommonOperationsMetadataOperation(xmlNsPtr psNsOws, xmlNsPtr psXLinkNs, char *name, int method, char *url);
+xmlNodePtr msOWSCommonOperationsMetadataOperation(xmlNsPtr psNsOws, xmlNsPtr psXLinkNs, const char *name, int method, const char *url);
 
-xmlNodePtr msOWSCommonOperationsMetadataDomainType(int version, xmlNsPtr psNsOws, char *elname, char *name, char *values);
+xmlNodePtr msOWSCommonOperationsMetadataDomainType(int version, xmlNsPtr psNsOws, const char *elname, const char *name, const char *values);
 
 xmlNodePtr msOWSCommonExceptionReport(xmlNsPtr psNsOws, int ows_version, const char *schemas_location, const char *version, const char *language, const char *exceptionCode, const char *locator, const char *ExceptionText);
 
@@ -190,4 +196,8 @@ int msOWSSchemaValidation(const char* xml_schema, const char* xml);
 
 int msOWSCommonNegotiateVersion(int requested_version, const int supported_versions[], int num_supported_versions);
 
+#ifdef __cplusplus
+} /* extern C */
+#endif
+
 #endif /* MAPOWSCOMMON_H */
