@@ -99,13 +99,14 @@ static int msGetClass_String( layerObj *layer, colorObj *color, const char *pixe
   /* -------------------------------------------------------------------- */
   /*      Loop over classes till we find a match.                         */
   /* -------------------------------------------------------------------- */
-  for(i= (firstClassToTry < 0 ) ? 0 : -1; i<layer->numclasses; i++) {
+  if( firstClassToTry >= layer->numclasses )
+      firstClassToTry = -1;
+  for(i=0; i<layer->numclasses; i++) {
 
-    int idx = i;
-    if( i < 0 )
-        idx = firstClassToTry;
-    else if( i == firstClassToTry )
-        continue;
+    const int idx = firstClassToTry < 0 ? i :
+                    i == 0 ? firstClassToTry :
+                    i <= firstClassToTry ? i - 1:
+                    i;
 
     /* check for correct classgroup, if set */
     if ( layer->class[idx]->group && layer->classgroup &&
@@ -616,7 +617,7 @@ void* msDrawRasterLayerLowOpenDataset(mapObj *map, layerObj *layer,
 
   msGDALInitialize();
 
-  if(layer->debug == MS_TRUE)
+  if(layer->debug)
     msDebug( "msDrawRasterLayerLow(%s): Filename is: %s\n", layer->name, filename);
 
   if( strncmp(filename, "<VRTDataset", strlen("<VRTDataset")) == 0 )
@@ -628,7 +629,7 @@ void* msDrawRasterLayerLowOpenDataset(mapObj *map, layerObj *layer,
     msDrawRasterBuildRasterPath(map, layer, filename, szPath);
     pszPath = szPath;
   }
-  if(layer->debug == MS_TRUE)
+  if(layer->debug)
     msDebug("msDrawRasterLayerLow(%s): Path is: %s\n", layer->name, pszPath);
 
     /*
@@ -703,27 +704,27 @@ void msDrawRasterLayerLowCloseDataset(layerObj *layer, void* hDS)
 
 int msDrawRasterLayerLowCheckIfMustDraw(mapObj *map, layerObj *layer)
 {
-  if(!layer->data && !layer->tileindex && !(layer->connectiontype==MS_KERNELDENSITY)) {
-    if(layer->debug == MS_TRUE)
+  if(!layer->data && !layer->tileindex && !(layer->connectiontype==MS_KERNELDENSITY || layer->connectiontype==MS_IDW)) {
+    if(layer->debug)
       msDebug( "msDrawRasterLayerLow(%s): layer data and tileindex NULL ... doing nothing.", layer->name );
     return(0);
   }
 
   if((layer->status != MS_ON) && (layer->status != MS_DEFAULT)) {
-    if(layer->debug == MS_TRUE)
+    if(layer->debug)
       msDebug( "msDrawRasterLayerLow(%s): not status ON or DEFAULT, doing nothing.", layer->name );
     return(0);
   }
 
   if(map->scaledenom > 0) {
     if((layer->maxscaledenom > 0) && (map->scaledenom > layer->maxscaledenom)) {
-      if(layer->debug == MS_TRUE)
+      if(layer->debug)
         msDebug( "msDrawRasterLayerLow(%s): skipping, map scale %.2g > MAXSCALEDENOM=%g\n",
                  layer->name, map->scaledenom, layer->maxscaledenom );
       return(0);
     }
     if((layer->minscaledenom > 0) && (map->scaledenom <= layer->minscaledenom)) {
-      if(layer->debug == MS_TRUE)
+      if(layer->debug)
         msDebug( "msDrawRasterLayerLow(%s): skipping, map scale %.2g < MINSCALEDENOM=%g\n",
                  layer->name, map->scaledenom, layer->minscaledenom );
       return(0);
@@ -732,13 +733,13 @@ int msDrawRasterLayerLowCheckIfMustDraw(mapObj *map, layerObj *layer)
 
   if(layer->maxscaledenom <= 0 && layer->minscaledenom <= 0) {
     if((layer->maxgeowidth > 0) && ((map->extent.maxx - map->extent.minx) > layer->maxgeowidth)) {
-      if(layer->debug == MS_TRUE)
+      if(layer->debug)
         msDebug( "msDrawRasterLayerLow(%s): skipping, map width %.2g > MAXSCALEDENOM=%g\n", layer->name,
                  (map->extent.maxx - map->extent.minx), layer->maxgeowidth );
       return(0);
     }
     if((layer->mingeowidth > 0) && ((map->extent.maxx - map->extent.minx) < layer->mingeowidth)) {
-      if(layer->debug == MS_TRUE)
+      if(layer->debug)
         msDebug( "msDrawRasterLayerLow(%s): skipping, map width %.2g < MINSCALEDENOM=%g\n", layer->name,
                  (map->extent.maxx - map->extent.minx), layer->mingeowidth );
       return(0);
@@ -832,9 +833,9 @@ int msDrawRasterLayerLowWithDataset(mapObj *map, layerObj *layer, imageObj *imag
       done = MS_TRUE; /* only one image so we're done after this */
     }
     
-    if(layer->connectiontype == MS_KERNELDENSITY) {
+    if(layer->connectiontype == MS_KERNELDENSITY || layer->connectiontype == MS_IDW) {
       msAcquireLock( TLOCK_GDAL );
-      status = msComputeKernelDensityDataset(map, image, layer, &hDS, &kernel_density_cleanup_ptr);
+      status = msInterpolationDataset(map, image, layer, &hDS, &kernel_density_cleanup_ptr);
       if(status != MS_SUCCESS) {
         msReleaseLock( TLOCK_GDAL );
         final_status = status;
@@ -942,7 +943,7 @@ int msDrawRasterLayerLowWithDataset(mapObj *map, layerObj *layer, imageObj *imag
     ** default to keeping open for single data files, and
     ** to closing for tile indexes
     */
-    if(layer->connectiontype == MS_KERNELDENSITY) {
+    if(layer->connectiontype == MS_KERNELDENSITY || layer->connectiontype == MS_IDW) {
       /*
       ** Fix issue #5330
       ** The in-memory kernel density heatmap gdal dataset handle (hDS) gets re-used
@@ -966,8 +967,8 @@ cleanup:
   if(layer->tileindex) { /* tiling clean-up */
     msDrawRasterCleanupTileLayer(tlp, tilelayerindex);
   }
-  if(layer->connectiontype == MS_KERNELDENSITY && kernel_density_cleanup_ptr) {
-    msCleanupKernelDensityDataset(map, image, layer, kernel_density_cleanup_ptr);
+  if(kernel_density_cleanup_ptr) {
+    msCleanupInterpolationDataset(map, image, layer, kernel_density_cleanup_ptr);
   }
 
   return final_status;
@@ -1005,12 +1006,16 @@ imageObj *msDrawReferenceMap(mapObj *map)
 
   image = msImageCreate(refImage->width, refImage->height, map->outputformat,
                         map->web.imagepath, map->web.imageurl, map->resolution, map->defresolution, &(map->reference.color));
-  if(!image) return NULL;
+  if(!image)
+  {
+      free(refImage);
+      return NULL;
+  }
 
   status = renderer->mergeRasterBuffer(image,refImage,1.0,0,0,0,0,refImage->width, refImage->height);
   msFreeRasterBuffer(refImage);
   free(refImage);
-  if(UNLIKELY(status == MS_FAILURE))
+  if(MS_UNLIKELY(status == MS_FAILURE))
     return NULL;
 
   /* make sure the extent given in mapfile fits the image */
@@ -1058,7 +1063,7 @@ imageObj *msDrawReferenceMap(mapObj *map)
     if( map->reference.maxboxsize == 0 ||
         ((abs(x2 - x1) < map->reference.maxboxsize) &&
          (abs(y2 - y1) < map->reference.maxboxsize)) ) {
-      if(UNLIKELY(MS_FAILURE == msDrawShadeSymbol(map, image, &rect, &style, 1.0))) {
+      if(MS_UNLIKELY(MS_FAILURE == msDrawShadeSymbol(map, image, &rect, &style, 1.0))) {
         msFreeImage(image);
         return NULL;
       }
@@ -1082,7 +1087,7 @@ imageObj *msDrawReferenceMap(mapObj *map)
           style.symbol = msGetSymbolIndex(&map->symbolset,  map->reference.markername, MS_TRUE);
         }
 
-        if(UNLIKELY(MS_FAILURE == msDrawMarkerSymbol(map, image, &point, &style, 1.0))) {
+        if(MS_UNLIKELY(MS_FAILURE == msDrawMarkerSymbol(map, image, &point, &style, 1.0))) {
           msFreeImage(image);
           return NULL;
         }
@@ -1121,7 +1126,7 @@ imageObj *msDrawReferenceMap(mapObj *map)
         cross.line[3].point[1].x = x21+8;
         cross.line[3].point[1].y = y21;
 
-        if(UNLIKELY(MS_FAILURE == msDrawLineSymbol(map,image,&cross,&style,1.0))) {
+        if(MS_UNLIKELY(MS_FAILURE == msDrawLineSymbol(map,image,&cross,&style,1.0))) {
           msFreeImage(image);
           return NULL;
         }
